"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DebugServer = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Copyright (c) 2017-present, Facebook, Inc.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * All rights reserved.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * This source code is licensed under the BSD-style license found in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * LICENSE file in the root directory of this source tree. An additional grant
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * of patent rights can be found in the PATENTS file in the same directory.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */

var _BreakpointCollection = require("./BreakpointCollection.js");

var _Breakpoint = require("./Breakpoint.js");

var _invariant = require("../invariant.js");

var _invariant2 = _interopRequireDefault(_invariant);

var _DebugMessage = require("./channel/DebugMessage.js");

var _DebuggerError = require("./DebuggerError.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DebugServer = exports.DebugServer = function () {
  function DebugServer(channel, realm) {
    _classCallCheck(this, DebugServer);

    this._breakpoints = new _BreakpointCollection.BreakpointCollection();
    this._previousExecutedLine = 0;
    this._previousExecutedCol = 0;
    this._lastRunRequestID = 0;
    this._channel = channel;
    this._realm = realm;
    this.waitForRun();
  }
  // the collection of breakpoints

  // the channel to communicate with the adapter


  _createClass(DebugServer, [{
    key: "waitForRun",


    /* Block until adapter says to run
    /* runCondition: a function that determines whether the adapter has told
    /* Prepack to continue running
    */
    value: function waitForRun() {
      var keepRunning = false;
      var request = void 0;
      while (!keepRunning) {
        request = this._channel.readIn();
        keepRunning = this.processDebuggerCommand(request);
      }
    }

    // Checking if the debugger needs to take any action on reaching this ast node

  }, {
    key: "checkForActions",
    value: function checkForActions(ast) {
      this.checkForBreakpoint(ast);
      // last step: set the current location as the previously executed line
      if (ast.loc && ast.loc.source !== null) {
        this._previousExecutedFile = ast.loc.source;
        this._previousExecutedLine = ast.loc.start.line;
        this._previousExecutedCol = ast.loc.start.column;
      }
    }

    // Try to find a breakpoint at the given location and check if we should stop on it

  }, {
    key: "findStoppableBreakpoint",
    value: function findStoppableBreakpoint(filePath, lineNum, colNum) {
      var breakpoint = this._breakpoints.getBreakpoint(filePath, lineNum, colNum);
      if (breakpoint && breakpoint.enabled) {
        // checking if this is the same file and line we stopped at last time
        // if so, we should skip it this time
        // Note: for the case when the debugger is supposed to stop on the same
        // breakpoint consecutively (e.g. the statement is in a loop), some other
        // ast node (e.g. block, loop) must have been checked in between so
        // previousExecutedFile and previousExecutedLine will have changed
        if (breakpoint.column !== 0) {
          // this is a column breakpoint
          if (filePath === this._previousExecutedFile && lineNum === this._previousExecutedLine && colNum === this._previousExecutedCol) {
            return null;
          }
        } else {
          // this is a line breakpoint
          if (filePath === this._previousExecutedFile && lineNum === this._previousExecutedLine) {
            return null;
          }
        }
        return breakpoint;
      }
      return null;
    }
  }, {
    key: "checkForBreakpoint",
    value: function checkForBreakpoint(ast) {
      if (ast.loc && ast.loc.source) {
        var location = ast.loc;
        var filePath = location.source;
        if (filePath === null) return;
        var lineNum = location.start.line;
        var colNum = location.start.column;
        // Check whether there is a breakpoint we need to stop on here
        var breakpoint = this.findStoppableBreakpoint(filePath, lineNum, colNum);
        if (breakpoint === null) return;
        // Tell the adapter that Prepack has stopped on this breakpoint
        this._channel.sendBreakpointStopped(breakpoint.filePath, breakpoint.line, breakpoint.column);
        // Wait for the adapter to tell us to run again
        this.waitForRun();
      }
    }

    // Process a command from a debugger. Returns whether Prepack should unblock
    // if it is blocked

  }, {
    key: "processDebuggerCommand",
    value: function processDebuggerCommand(request) {
      var requestID = request.id;
      var command = request.command;
      var args = request.arguments;
      switch (command) {
        case _DebugMessage.DebugMessage.BREAKPOINT_ADD_COMMAND:
          (0, _invariant2.default)(args.kind === "breakpoint");
          this._breakpoints.addBreakpoint(args.filePath, args.line, args.column);
          this._channel.sendBreakpointAcknowledge(_DebugMessage.DebugMessage.BREAKPOINT_ADD_ACKNOWLEDGE, requestID, args);
          break;
        case _DebugMessage.DebugMessage.BREAKPOINT_REMOVE_COMMAND:
          (0, _invariant2.default)(args.kind === "breakpoint");
          this._breakpoints.removeBreakpoint(args.filePath, args.line, args.column);
          this._channel.sendBreakpointAcknowledge(_DebugMessage.DebugMessage.BREAKPOINT_REMOVE_ACKNOWLEDGE, requestID, args);
          break;
        case _DebugMessage.DebugMessage.BREAKPOINT_ENABLE_COMMAND:
          (0, _invariant2.default)(args.kind === "breakpoint");
          this._breakpoints.enableBreakpoint(args.filePath, args.line, args.column);
          this._channel.sendBreakpointAcknowledge(_DebugMessage.DebugMessage.BREAKPOINT_ENABLE_ACKNOWLEDGE, requestID, args);
          break;
        case _DebugMessage.DebugMessage.BREAKPOINT_DISABLE_COMMAND:
          (0, _invariant2.default)(args.kind === "breakpoint");
          this._breakpoints.disableBreakpoint(args.filePath, args.line, args.column);
          this._channel.sendBreakpointAcknowledge(_DebugMessage.DebugMessage.BREAKPOINT_DISABLE_ACKNOWLEDGE, requestID, args);
          break;
        case _DebugMessage.DebugMessage.PREPACK_RUN_COMMAND:
          (0, _invariant2.default)(args.kind === "run");
          return true;
        case _DebugMessage.DebugMessage.STACKFRAMES_COMMAND:
          (0, _invariant2.default)(args.kind === "stackframe");
          this.processStackframesCommand(requestID, args);
          break;
        default:
          throw new _DebuggerError.DebuggerError("Invalid command", "Invalid command from adapter: " + command);
      }
      return false;
    }
  }, {
    key: "processStackframesCommand",
    value: function processStackframesCommand(requestID, args) {
      var frameInfos = [];
      var loc = this._realm.currentLocation;

      // the UI displays the current frame as index 0, so we iterate backwards
      // from the current frame
      for (var i = this._realm.contextStack.length - 1; i >= 0; i--) {
        var frame = this._realm.contextStack[i];
        var functionName = "(anonymous function)";
        if (frame.function && frame.function.__originalName) {
          functionName = frame.function.__originalName;
        }
        var fileName = "unknown";
        var line = 0;
        var column = 0;
        if (loc && loc.source) {
          fileName = loc.source;
          line = loc.start.line;
          column = loc.start.column;
        }
        var frameInfo = {
          id: this._realm.contextStack.length - 1 - i,
          functionName: functionName,
          fileName: fileName,
          line: line,
          column: column
        };
        frameInfos.push(frameInfo);
        loc = frame.loc;
      }
      this._channel.sendStackframeResponse(requestID, frameInfos);
    }
  }, {
    key: "shutdown",
    value: function shutdown() {
      //let the adapter know Prepack is done running
      this._channel.sendPrepackFinish();
    }
  }]);

  return DebugServer;
}();
//# sourceMappingURL=Debugger.js.map