"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _vscodeDebugadapter = require("vscode-debugadapter");

var _vscodeDebugprotocol = require("vscode-debugprotocol");

var DebugProtocol = _interopRequireWildcard(_vscodeDebugprotocol);

var _child_process = require("child_process");

var _child_process2 = _interopRequireDefault(_child_process);

var _AdapterChannel = require("./../channel/AdapterChannel.js");

var _prepackOptions = require("./../../prepack-options.js");

var _invariant = require("./../../invariant.js");

var _invariant2 = _interopRequireDefault(_invariant);

var _DebugMessage = require("./../channel/DebugMessage.js");

var _DebuggerConstants = require("./../DebuggerConstants.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright (c) 2017-present, Facebook, Inc.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * All rights reserved.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This source code is licensed under the BSD-style license found in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * LICENSE file in the root directory of this source tree. An additional grant
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * of patent rights can be found in the PATENTS file in the same directory.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */

/* An implementation of an debugger adapter adhering to the VSCode Debug protocol
 * The adapter is responsible for communication between the UI and Prepack
*/
var PrepackDebugSession = function (_LoggingDebugSession) {
  _inherits(PrepackDebugSession, _LoggingDebugSession);

  /**
   * Creates a new debug adapter that is used for one debug session.
   * We configure the default implementation of a debug adapter here.
   */
  function PrepackDebugSession() {
    _classCallCheck(this, PrepackDebugSession);

    var _this = _possibleConstructorReturn(this, (PrepackDebugSession.__proto__ || Object.getPrototypeOf(PrepackDebugSession)).call(this, "prepack"));

    _this.setDebuggerLinesStartAt1(true);
    _this.setDebuggerColumnsStartAt1(true);
    _this._pendingRequestCallbacks = new Map();
    _this._readCLIParameters();
    _this._startPrepack();
    return _this;
  }

  _createClass(PrepackDebugSession, [{
    key: "_readCLIParameters",
    value: function _readCLIParameters() {
      var args = Array.from(process.argv);
      args.splice(0, 2);
      var inFilePath = void 0;
      var outFilePath = void 0;
      while (args.length > 0) {
        var arg = args.shift();
        if (arg.startsWith("--")) {
          arg = arg.slice(2);
          if (arg === "prepack") {
            this._prepackCommand = args.shift();
          } else if (arg === "inFilePath") {
            inFilePath = args.shift();
          } else if (arg === "outFilePath") {
            outFilePath = args.shift();
          }
        } else {
          console.error("Unknown parameter: " + arg);
          process.exit(1);
        }
      }
      if (!inFilePath || inFilePath.length === 0) {
        console.error("No debugger input file given");
        process.exit(1);
      }
      if (!outFilePath || outFilePath.length === 0) {
        console.error("No debugger output file given");
        process.exit(1);
      }
      this._debuggerOptions = (0, _prepackOptions.getDebuggerOptions)({
        debugInFilePath: inFilePath,
        debugOutFilePath: outFilePath
      });
    }

    // Start Prepack in a child process

  }, {
    key: "_startPrepack",
    value: function _startPrepack() {
      var _this2 = this;

      if (!this._prepackCommand || this._prepackCommand.length === 0) {
        console.error("No command given to start Prepack in adapter");
        process.exit(1);
      }

      // set up the communication channel
      this._adapterChannel = new _AdapterChannel.AdapterChannel(this._debuggerOptions);
      this._registerMessageCallbacks();

      var prepackArgs = this._prepackCommand.split(" ");
      // Note: here the input file for the adapter is the output file for Prepack, and vice versa.
      prepackArgs = prepackArgs.concat(["--debugInFilePath", this._debuggerOptions.outFilePath, "--debugOutFilePath", this._debuggerOptions.inFilePath]);
      this._prepackProcess = _child_process2.default.spawn("node", prepackArgs);

      process.on("exit", function () {
        _this2._prepackProcess.kill();
        _this2._adapterChannel.clean();
        process.exit();
      });

      process.on("SIGINT", function () {
        _this2._prepackProcess.kill();
        process.exit();
      });

      this._prepackProcess.stdout.on("data", function (data) {
        var outputEvent = new _vscodeDebugadapter.OutputEvent(data.toString(), "stdout");
        _this2.sendEvent(outputEvent);
      });

      this._prepackProcess.on("exit", function () {
        _this2.sendEvent(new _vscodeDebugadapter.TerminatedEvent());
        process.exit();
      });
    }
  }, {
    key: "_registerMessageCallbacks",
    value: function _registerMessageCallbacks() {
      var _this3 = this;

      this._adapterChannel.registerChannelEvent(_DebugMessage.DebugMessage.PREPACK_READY_RESPONSE, function (response) {
        _this3.sendEvent(new _vscodeDebugadapter.StoppedEvent("entry", _DebuggerConstants.DebuggerConstants.PREPACK_THREAD_ID));
      });
      this._adapterChannel.registerChannelEvent(_DebugMessage.DebugMessage.BREAKPOINT_STOPPED_RESPONSE, function (response) {
        var result = response.result;
        (0, _invariant2.default)(result.kind === "breakpoint-stopped");
        _this3.sendEvent(new _vscodeDebugadapter.StoppedEvent("breakpoint " + (result.filePath + " " + result.line + ":" + result.column), _DebuggerConstants.DebuggerConstants.PREPACK_THREAD_ID));
      });
    }
  }, {
    key: "_addRequestCallback",
    value: function _addRequestCallback(requestID, callback) {
      (0, _invariant2.default)(!(requestID in this._pendingRequestCallbacks), "Request ID already exists in pending requests");
      this._pendingRequestCallbacks[requestID] = callback;
    }

    /**
     * The 'initialize' request is the first request called by the UI
     * to interrogate the features the debug adapter provides.
     */

  }, {
    key: "initializeRequest",
    value: function initializeRequest(response, args) {
      // Let the UI know that we can start accepting breakpoint requests.
      // The UI will end the configuration sequence by calling 'configurationDone' request.
      this.sendEvent(new _vscodeDebugadapter.InitializedEvent());

      response.body = response.body || {};
      response.body.supportsConfigurationDoneRequest = true;
      // Respond back to the UI with the configurations. Will add more configurations gradually as needed.
      // Adapter can respond immediately here because no message is sent to Prepack
      this.sendResponse(response);
    }

    /**
     * Request Prepack to continue running when it is stopped
    */

  }, {
    key: "continueRequest",
    value: function continueRequest(response, args) {
      var _this4 = this;

      // send a Run request to Prepack and try to send the next request
      this._adapterChannel.run(response.request_seq, function (dbgResponse) {
        _this4.sendResponse(response);
      });
    }
  }, {
    key: "setBreakPointsRequest",
    value: function setBreakPointsRequest(response, args) {
      var _this5 = this;

      if (!args.source.path || !args.breakpoints) return;
      var filePath = args.source.path;
      var breakpointInfos = [];
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = args.breakpoints[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var breakpoint = _step.value;

          var line = breakpoint.line;
          var column = 0;
          if (breakpoint.column) {
            column = breakpoint.column;
          }
          var breakpointInfo = {
            kind: "breakpoint",
            requestID: response.request_seq,
            filePath: filePath,
            line: line,
            column: column
          };
          breakpointInfos.push(breakpointInfo);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      this._adapterChannel.setBreakpoints(response.request_seq, breakpointInfos, function (dbgResponse) {
        _this5.sendResponse(response);
      });
    }
  }, {
    key: "stackTraceRequest",
    value: function stackTraceRequest(response, args) {
      var _this6 = this;

      this._adapterChannel.getStackFrames(response.request_seq, function (dbgResponse) {
        var result = dbgResponse.result;
        (0, _invariant2.default)(result.kind === "stackframe");
        var frameInfos = result.stackframes;
        var frames = [];
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = frameInfos[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var frameInfo = _step2.value;

            var source = {
              path: frameInfo.fileName
            };
            var frame = {
              id: frameInfo.id,
              name: frameInfo.functionName,
              source: source,
              line: frameInfo.line,
              column: frameInfo.column
            };
            frames.push(frame);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        response.body = {
          stackFrames: frames
        };
        _this6.sendResponse(response);
      });
    }
  }, {
    key: "threadsRequest",
    value: function threadsRequest(response) {
      // There will only be 1 thread, so respond immediately
      var thread = {
        id: _DebuggerConstants.DebuggerConstants.PREPACK_THREAD_ID,
        name: "main"
      };
      response.body = {
        threads: [thread]
      };
      this.sendResponse(response);
    }
  }]);

  return PrepackDebugSession;
}(_vscodeDebugadapter.LoggingDebugSession);

_vscodeDebugadapter.DebugSession.run(PrepackDebugSession);
//# sourceMappingURL=DebugAdapter.js.map