"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AdapterChannel = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Copyright (c) 2017-present, Facebook, Inc.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * All rights reserved.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * This source code is licensed under the BSD-style license found in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * LICENSE file in the root directory of this source tree. An additional grant
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * of patent rights can be found in the PATENTS file in the same directory.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */

var _FileIOWrapper = require("./FileIOWrapper.js");

var _MessageMarshaller = require("./MessageMarshaller.js");

var _queueFifo = require("queue-fifo");

var _queueFifo2 = _interopRequireDefault(_queueFifo);

var _events = require("events");

var _events2 = _interopRequireDefault(_events);

var _invariant = require("./../../invariant.js");

var _invariant2 = _interopRequireDefault(_invariant);

var _DebugMessage = require("./DebugMessage.js");

var _DebuggerConstants = require("./../DebuggerConstants.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

//Channel used by the debug adapter to communicate with Prepack
var AdapterChannel = exports.AdapterChannel = function () {
  function AdapterChannel(dbgOptions) {
    _classCallCheck(this, AdapterChannel);

    this._ioWrapper = new _FileIOWrapper.FileIOWrapper(true, dbgOptions.inFilePath, dbgOptions.outFilePath);
    this._marshaller = new _MessageMarshaller.MessageMarshaller();
    this._queue = new _queueFifo2.default();
    this._pendingRequestCallbacks = new Map();
    this._eventEmitter = new _events2.default();
    this.sendDebuggerStart(_DebuggerConstants.DebuggerConstants.DEFAULT_REQUEST_ID);
    this.listenOnFile(this._processPrepackMessage.bind(this));
  }

  _createClass(AdapterChannel, [{
    key: "_handleFileReadError",


    // Error handler for errors in files from the adapter channel
    value: function _handleFileReadError(err) {
      console.error(err);
      process.exit(1);
    }
  }, {
    key: "_processPrepackMessage",
    value: function _processPrepackMessage(message) {
      var parts = message.split(" ");
      var requestID = parseInt(parts[0], 10);
      (0, _invariant2.default)(!isNaN(requestID));
      var messageType = parts[1];
      if (messageType === _DebugMessage.DebugMessage.PREPACK_READY_RESPONSE) {
        var result = this._marshaller.unmarshallReadyResponse(requestID);
        this._prepackWaiting = true;
        this._eventEmitter.emit(_DebugMessage.DebugMessage.PREPACK_READY_RESPONSE, result);
        this.trySendNextRequest();
      } else if (messageType === _DebugMessage.DebugMessage.BREAKPOINT_ADD_ACKNOWLEDGE) {
        var dbgResponse = this._marshaller.unmarshallBreakpointAddResponse(requestID);
        this._eventEmitter.emit(_DebugMessage.DebugMessage.BREAKPOINT_ADD_ACKNOWLEDGE, requestID, dbgResponse);
        // Prepack acknowledged adding a breakpoint
        this._prepackWaiting = true;
        this._processRequestCallback(requestID, dbgResponse);
        this.trySendNextRequest();
      } else if (messageType === _DebugMessage.DebugMessage.BREAKPOINT_STOPPED_RESPONSE) {
        var _dbgResponse = this._marshaller.unmarshallBreakpointStoppedResponse(requestID, parts.slice(2));
        this._eventEmitter.emit(_DebugMessage.DebugMessage.BREAKPOINT_STOPPED_RESPONSE, _dbgResponse);
        // Prepack stopped on a breakpoint
        this._prepackWaiting = true;
        this._processRequestCallback(requestID, _dbgResponse);
        this.trySendNextRequest();
      } else if (messageType === _DebugMessage.DebugMessage.STACKFRAMES_RESPONSE) {
        var _dbgResponse2 = this._marshaller.unmarshallStackframesResponse(requestID, parts.slice(2).join(" "));
        this._prepackWaiting = true;
        this._processRequestCallback(requestID, _dbgResponse2);
        this.trySendNextRequest();
      }
    }

    // Check to see if the next request to Prepack can be sent and send it if so

  }, {
    key: "trySendNextRequest",
    value: function trySendNextRequest() {
      // check to see if Prepack is ready to accept another request
      if (!this._prepackWaiting) return false;
      // check that there is a message to send
      if (this._queue.isEmpty()) return false;
      var request = this._queue.dequeue();
      this.listenOnFile(this._processPrepackMessage.bind(this));
      this.writeOut(request);
      this._prepackWaiting = false;
      return true;
    }
  }, {
    key: "_addRequestCallback",
    value: function _addRequestCallback(requestID, callback) {
      (0, _invariant2.default)(!(requestID in this._pendingRequestCallbacks), "Request ID already exists in pending requests");
      this._pendingRequestCallbacks[requestID] = callback;
    }
  }, {
    key: "_processRequestCallback",
    value: function _processRequestCallback(requestID, response) {
      (0, _invariant2.default)(requestID in this._pendingRequestCallbacks, "Request ID does not exist in pending requests: " + requestID);
      var callback = this._pendingRequestCallbacks[requestID];
      callback(response);
    }
  }, {
    key: "registerChannelEvent",
    value: function registerChannelEvent(event, listener) {
      this._eventEmitter.addListener(event, listener);
    }
  }, {
    key: "run",
    value: function run(requestID, callback) {
      this._queue.enqueue(this._marshaller.marshallContinueRequest(requestID));
      this.trySendNextRequest();
      this._addRequestCallback(requestID, callback);
    }
  }, {
    key: "setBreakpoints",
    value: function setBreakpoints(requestID, breakpoints, callback) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = breakpoints[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var breakpoint = _step.value;

          this._queue.enqueue(this._marshaller.marshallSetBreakpointsRequest(requestID, breakpoint));
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      this.trySendNextRequest();
      this._addRequestCallback(requestID, callback);
    }
  }, {
    key: "getStackFrames",
    value: function getStackFrames(requestID, callback) {
      this._queue.enqueue(this._marshaller.marshallStackFramesRequest(requestID));
      this.trySendNextRequest();
      this._addRequestCallback(requestID, callback);
    }
  }, {
    key: "writeOut",
    value: function writeOut(contents) {
      this._ioWrapper.writeOutSync(contents);
    }
  }, {
    key: "sendDebuggerStart",
    value: function sendDebuggerStart(requestID) {
      this.writeOut(this._marshaller.marshallDebuggerStart(requestID));
    }
  }, {
    key: "listenOnFile",
    value: function listenOnFile(messageProcessor) {
      this._ioWrapper.readIn(this._handleFileReadError.bind(this), messageProcessor);
    }
  }, {
    key: "clean",
    value: function clean() {
      this._ioWrapper.clearInFile();
      this._ioWrapper.clearOutFile();
    }
  }]);

  return AdapterChannel;
}();
//# sourceMappingURL=AdapterChannel.js.map