"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MessageMarshaller = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Copyright (c) 2017-present, Facebook, Inc.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * All rights reserved.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * This source code is licensed under the BSD-style license found in the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * LICENSE file in the root directory of this source tree. An additional grant
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * of patent rights can be found in the PATENTS file in the same directory.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */

var _DebugMessage = require("./DebugMessage.js");

var _invariant = require("./../../invariant.js");

var _invariant2 = _interopRequireDefault(_invariant);

var _DebuggerError = require("./../DebuggerError.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var MessageMarshaller = exports.MessageMarshaller = function () {
  function MessageMarshaller() {
    _classCallCheck(this, MessageMarshaller);
  }

  _createClass(MessageMarshaller, [{
    key: "marshallBreakpointAcknowledge",
    value: function marshallBreakpointAcknowledge(requestID, prefix, filePath, line, column) {
      return requestID + " " + prefix + " " + filePath + " " + line + " " + column;
    }
  }, {
    key: "marshallBreakpointStopped",
    value: function marshallBreakpointStopped(requestID, args) {
      return requestID + " " + _DebugMessage.DebugMessage.BREAKPOINT_STOPPED_RESPONSE + " " + args.filePath + " " + args.line + " " + args.column;
    }
  }, {
    key: "marshallPrepackFinish",
    value: function marshallPrepackFinish(requestID) {
      return requestID + " " + _DebugMessage.DebugMessage.PREPACK_FINISH_RESPONSE;
    }
  }, {
    key: "marshallDebuggerStart",
    value: function marshallDebuggerStart(requestID) {
      return requestID + " " + _DebugMessage.DebugMessage.DEBUGGER_ATTACHED;
    }
  }, {
    key: "marshallContinueRequest",
    value: function marshallContinueRequest(requestID) {
      return requestID + " " + _DebugMessage.DebugMessage.PREPACK_RUN_COMMAND;
    }
  }, {
    key: "marshallSetBreakpointsRequest",
    value: function marshallSetBreakpointsRequest(requestID, args) {
      return requestID + " " + _DebugMessage.DebugMessage.BREAKPOINT_ADD_COMMAND + " " + args.filePath + " " + args.line + " " + args.column;
    }
  }, {
    key: "marshallStackFramesRequest",
    value: function marshallStackFramesRequest(requestID) {
      return requestID + " " + _DebugMessage.DebugMessage.STACKFRAMES_COMMAND;
    }
  }, {
    key: "marshallStackFramesResponse",
    value: function marshallStackFramesResponse(requestID, stackframes) {
      return requestID + " " + _DebugMessage.DebugMessage.STACKFRAMES_RESPONSE + " " + JSON.stringify(stackframes);
    }
  }, {
    key: "unmarshallBreakpointArguments",
    value: function unmarshallBreakpointArguments(requestID, parts) {
      var filePath = parts[0];

      var lineNum = parseInt(parts[1], 10);
      (0, _invariant2.default)(!isNaN(lineNum));
      var columnNum = 0;
      if (parts.length === 3) {
        columnNum = parseInt(parts[2], 10);
        (0, _invariant2.default)(!isNaN(columnNum));
      }

      var result = {
        kind: "breakpoint",
        filePath: filePath,
        line: lineNum,
        column: columnNum
      };
      return result;
    }
  }, {
    key: "unmarshallStackframesResponse",
    value: function unmarshallStackframesResponse(requestID, responseBody) {
      try {
        var frames = JSON.parse(responseBody);
        (0, _invariant2.default)(Array.isArray(frames), "Stack frames is not an array");
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = frames[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var frame = _step.value;

            (0, _invariant2.default)(frame.hasOwnProperty("id"), "Stack frame is missing id");
            (0, _invariant2.default)(frame.hasOwnProperty("fileName"), "Stack frame is missing filename");
            (0, _invariant2.default)(frame.hasOwnProperty("line"), "Stack frame is missing line number");
            (0, _invariant2.default)(frame.hasOwnProperty("column"), "Stack frame is missing column number");
            (0, _invariant2.default)(frame.hasOwnProperty("functionName"), "Stack frame is missing function name");
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        var result = {
          kind: "stackframe",
          stackframes: frames
        };
        var dbgResponse = {
          id: requestID,
          result: result
        };
        return dbgResponse;
      } catch (e) {
        throw new _DebuggerError.DebuggerError("Invalid response", e.message);
      }
    }
  }, {
    key: "unmarshallBreakpointAddResponse",
    value: function unmarshallBreakpointAddResponse(requestID) {
      var result = {
        kind: "breakpoint-add"
      };
      var dbgResponse = {
        id: requestID,
        result: result
      };
      return dbgResponse;
    }
  }, {
    key: "unmarshallBreakpointStoppedResponse",
    value: function unmarshallBreakpointStoppedResponse(requestID, parts) {
      (0, _invariant2.default)(parts.length === 3, "Incorrect number of arguments in breakpoint stopped response");
      var filePath = parts[0];
      var line = parseInt(parts[1], 10);
      (0, _invariant2.default)(!isNaN(line), "Invalid line number");
      var column = parseInt(parts[2], 10);
      (0, _invariant2.default)(!isNaN(column), "Invalid column number");
      var result = {
        kind: "breakpoint-stopped",
        filePath: filePath,
        line: line,
        column: column
      };
      var dbgResponse = {
        id: requestID,
        result: result
      };
      return dbgResponse;
    }
  }, {
    key: "unmarshallReadyResponse",
    value: function unmarshallReadyResponse(requestID) {
      var result = {
        kind: "ready"
      };
      var dbgResponse = {
        id: requestID,
        result: result
      };
      return dbgResponse;
    }
  }]);

  return MessageMarshaller;
}();
//# sourceMappingURL=MessageMarshaller.js.map