"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (ast, strictCode, env, realm) {
  var completions = [];

  var blockRes = env.evaluateCompletionDeref(ast.block, strictCode);
  blockRes = (0, _index.incorporateSavedCompletion)(realm, blockRes);
  if (blockRes instanceof _completions.PossiblyNormalCompletion) {
    // The current state may have advanced since the time control forked into the various paths recorded in blockRes.
    // Update the normal path and restore the global state to what it was at the time of the fork.
    var subsequentEffects = realm.getCapturedEffects(blockRes.value);
    (0, _invariant2.default)(subsequentEffects !== undefined);
    realm.stopEffectCaptureAndUndoEffects();
    (0, _index.updatePossiblyNormalCompletionWithSubsequentEffects)(realm, blockRes, subsequentEffects);
  }

  if (ast.handler) {
    if (blockRes instanceof _completions.ThrowCompletion) {
      blockRes = env.evaluateCompletionDeref(ast.handler, strictCode, blockRes);
    } else if (blockRes instanceof _completions.JoinedAbruptCompletions || blockRes instanceof _completions.PossiblyNormalCompletion) {
      var handlerEffects = composeNestedThrowEffectsWithHandler(blockRes);
      blockRes = handlerEffects[0];
      // If there is a normal execution path following the handler, we need to update the current state
      if (blockRes instanceof _index2.Value || blockRes instanceof _completions.PossiblyNormalCompletion) realm.applyEffects(handlerEffects);
    }
  }
  completions.unshift(blockRes);

  if (ast.finalizer) {
    if (blockRes instanceof _completions.PossiblyNormalCompletion || blockRes instanceof _completions.JoinedAbruptCompletions) {
      completions.unshift(composeNestedEffectsWithFinalizer(blockRes));
    } else {
      completions.unshift(env.evaluateCompletion(ast.finalizer, strictCode));
    }
  }

  // Restart effect capture if one of the paths may continue
  if (blockRes instanceof _completions.PossiblyNormalCompletion) realm.captureEffects();

  // use the last completion record
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = completions[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var completion = _step.value;

      if (completion instanceof _completions.AbruptCompletion) throw completion;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  if (ast.finalizer) {
    completions.shift();
  }

  // otherwise use the last returned value
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = completions[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var _completion = _step2.value;

      if (_completion instanceof _completions.PossiblyNormalCompletion) _completion = realm.getRunningContext().composeWithSavedCompletion(_completion);
      if (_completion instanceof _index2.Value) return (0, _index.UpdateEmpty)(realm, _completion, realm.intrinsics.undefined);
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  (0, _invariant2.default)(false);

  // The finalizer is not a join point, so update each path in the completion separately.
  function composeNestedEffectsWithFinalizer(c) {
    var priorEffects = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    priorEffects.push(c.consequentEffects);
    var consequent = c.consequent;
    if (consequent instanceof _completions.PossiblyNormalCompletion || consequent instanceof _completions.JoinedAbruptCompletions) {
      composeNestedEffectsWithFinalizer(consequent, priorEffects);
    } else {
      c.consequentEffects = realm.evaluateForEffects(function () {
        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = priorEffects[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var priorEffect = _step3.value;
            realm.applyEffects(priorEffect);
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
              _iterator3.return();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }

        (0, _invariant2.default)(ast.finalizer);
        return env.evaluateCompletionDeref(ast.finalizer, strictCode);
      });
      var fc = c.consequentEffects[0];
      // If the finalizer had an abrupt completion, it overrides the try-block's completion.
      if (fc instanceof _completions.AbruptCompletion) c.consequent = fc;else c.consequentEffects[0] = consequent;
    }
    priorEffects.pop();
    priorEffects.push(c.alternateEffects);
    var alternate = c.alternate;
    if (alternate instanceof _completions.PossiblyNormalCompletion || alternate instanceof _completions.JoinedAbruptCompletions) {
      composeNestedEffectsWithFinalizer(alternate, priorEffects);
    } else {
      c.alternateEffects = realm.evaluateForEffects(function () {
        var _iteratorNormalCompletion4 = true;
        var _didIteratorError4 = false;
        var _iteratorError4 = undefined;

        try {
          for (var _iterator4 = priorEffects[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
            var priorEffect = _step4.value;
            realm.applyEffects(priorEffect);
          }
        } catch (err) {
          _didIteratorError4 = true;
          _iteratorError4 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion4 && _iterator4.return) {
              _iterator4.return();
            }
          } finally {
            if (_didIteratorError4) {
              throw _iteratorError4;
            }
          }
        }

        (0, _invariant2.default)(ast.finalizer);
        return env.evaluateCompletionDeref(ast.finalizer, strictCode);
      });
      var _fc = c.alternateEffects[0];
      // If the finalizer had an abrupt completion, it overrides the try-block's completion.
      if (_fc instanceof _completions.AbruptCompletion) c.alternate = _fc;else c.alternateEffects[0] = alternate;
    }
  }

  // The handler is a potential join point for all throw completions, but is easier to not do the join here because
  // it is tricky to join the joined and composed result of the throw completions with the non exceptional completions.
  // Unfortunately, things are still complicated because the handler may turn abrupt completions into normal
  // completions and the other way around. When this happens the container has to change its type.
  // We do this by call joinEffects to create a new container at every level of the recursion.
  function composeNestedThrowEffectsWithHandler(c) {
    var priorEffects = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    var consequent = c.consequent;
    var consequentEffects = c.consequentEffects;
    priorEffects.push(consequentEffects);
    if (consequent instanceof _completions.JoinedAbruptCompletions || consequent instanceof _completions.PossiblyNormalCompletion) {
      consequentEffects = composeNestedThrowEffectsWithHandler(consequent, priorEffects);
    } else if (consequent instanceof _completions.ThrowCompletion) {
      consequentEffects = realm.evaluateForEffects(function () {
        var _iteratorNormalCompletion5 = true;
        var _didIteratorError5 = false;
        var _iteratorError5 = undefined;

        try {
          for (var _iterator5 = priorEffects[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
            var priorEffect = _step5.value;
            realm.applyEffects(priorEffect);
          }
        } catch (err) {
          _didIteratorError5 = true;
          _iteratorError5 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion5 && _iterator5.return) {
              _iterator5.return();
            }
          } finally {
            if (_didIteratorError5) {
              throw _iteratorError5;
            }
          }
        }

        (0, _invariant2.default)(ast.handler);
        return env.evaluateCompletionDeref(ast.handler, strictCode, consequent);
      });
    }
    priorEffects.pop();
    var alternate = c.alternate;
    var alternateEffects = c.alternateEffects;
    priorEffects.push(alternateEffects);
    if (alternate instanceof _completions.PossiblyNormalCompletion || alternate instanceof _completions.JoinedAbruptCompletions) {
      alternateEffects = composeNestedThrowEffectsWithHandler(alternate, priorEffects);
    } else if (alternate instanceof _completions.ThrowCompletion) {
      alternateEffects = realm.evaluateForEffects(function () {
        var _iteratorNormalCompletion6 = true;
        var _didIteratorError6 = false;
        var _iteratorError6 = undefined;

        try {
          for (var _iterator6 = priorEffects[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
            var priorEffect = _step6.value;
            realm.applyEffects(priorEffect);
          }
        } catch (err) {
          _didIteratorError6 = true;
          _iteratorError6 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion6 && _iterator6.return) {
              _iterator6.return();
            }
          } finally {
            if (_didIteratorError6) {
              throw _iteratorError6;
            }
          }
        }

        (0, _invariant2.default)(ast.handler);
        return env.evaluateCompletionDeref(ast.handler, strictCode, alternate);
      });
    }
    priorEffects.pop();
    return (0, _index.joinEffects)(realm, c.joinCondition, consequentEffects, alternateEffects);
  }
};

require("../environment.js");

var _completions = require("../completions.js");

var _index = require("../methods/index.js");

var _index2 = require("../values/index.js");

var _invariant = require("../invariant.js");

var _invariant2 = _interopRequireDefault(_invariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
//# sourceMappingURL=TryStatement.js.map